/*----------------------------------------------------------------------------------
*
* COPYRIGHT (c) 2001 by Singing Electrons, Inc. All rights reserved.
*
*
* Module Name			: C:\se\adi\hidclass\Source\HidClass.c
*
* Description			: The HID Class implementation - contains routines that implment the HID Class device
*
*
* Revision History	: At bottom of the file.
*
*---------------------------------------------------------------------------------*/

/** include files **/
#include <sys/exception.h>
#include <sys/excause.h>
#include <defBF535.h>
#include "dtype.h"
#include "usb.h"
#include "usbdriver.h"
#include "hidclass.h"
#include "dprintf.h"

/** local definitions **/



/* default settings */
#define CONTROL_PIPE_MAX_TRANSFER_LENGTH 32
//#define VENDOR_ID 0xCB00
//#define PRODUCT_ID 0x0005
//#define DEVICE_RELEASE 0x0004 //0.04

#define INTERFACE_FOR_HID 0
#define ALTSETTING_FOR_HID 0

#define NUM_HID_DESCRIPTORS 1

#define EP1_MAX_PACKET_SIZE 8
#define EP1_POLLING_INTERVAL 10 //milliseconds



//
// String Descriptors		
//
enum {
	STRING_DEVICE = 1,
	STRING_PRODUCT,
	TOTAL_STRING_DESCRIPTORS
};

UCHAR gTheLangIDArray[] =
	{
		0x04,//bLength
		USB_STRING_DESCRIPTOR, //bDescriptorType
		0x09, 0x04 // 0x409 - English!
	};

UCHAR *gTheStringDescriptors[] =
	{
		"Analog Devices",
		"BlackFin HID Class Reference Implementation",
	};


#define OFFSET_VENDOR_ID 0x08
#define OFFSET_PRODUCT_ID 0x0A
#define OFFSET_RELEASE 0x0C
#define OFFSET_REPORT_DESCRIPTOR_LENGTH 25

//
// The Device Descriptor
//
UCHAR gTheDeviceDescriptor[] =
{
	0x12, //Length
	USB_DEVICE_DESCRIPTOR, //bDescriptorType
	0x10, 0x01, //bcdUSB - USB Spec 1.10
	0x00, //bDevice Class
	0x00, //bDeviceSubClass
	0x00, //bDeviceProtocol
	CONTROL_PIPE_PACKET_SIZE, //bMaxPacketSize0
	0x00, 0x00, //LOBYTE(VENDOR_ID), HIBYTE(VENDOR_ID),//idvendor
	0x00, 0x00, //LOBYTE(PRODUCT_ID), HIBYTE(PRODUCT_ID),//idProduct
	0x00, 0x00, //LOBYTE(DEVICE_RELEASE), HIBYTE(DEVICE_RELEASE),//bcdDevice
	STRING_DEVICE,//iManufaturer
	STRING_PRODUCT,//iProduct
	0,//iSerialNumber
	1//bNumConfigurations
};


//
// The Configuration Descriptor
//
UCHAR gTheConfigDescriptorWithINEndpoint[] =
	{
	0x09,			// Configuration Descriptor Length
	CONFIG_DESCR,	// Descriptor Type
	0x09 + 0x09 + 0x09 + 0x07,	// Configuration + Iface + HID + EPs length (LSB)
	0x00,			// Configuration+EPs length (MSB)
	0x01,			// # of interfaces
	0x01,			// configurationValue
	0x00,			// Configuration String
	0x40,			// Attributes: Self power and RWU
	0x00,     // max Power

    // Interface descriptor
    0x09, // length
    USB_INTERFACE_DESCRIPTOR,	// descriptorType
    INTERFACE_FOR_HID,		        // interfaceNumber
    ALTSETTING_FOR_HID,		        // alternateSetting
    1,		        // numEndpoints
    USB_HID_CLASS,		        // interfaceClass
    USB_HID_SUBCLASS_NONE,		        // interfaceSubClass
    USB_HID_PROTOCOL_NONE,		        // interfaceProtocol
    0,		        // interface

			//HID Descriptor
			0x09,//bLength
			USB_HID_DESCRIPTOR_HID,//bDescriptorType
			0x10, 0x01,//bcdHID - class spec. release
			USB_HID_COUNTRY_NONE,//bCountryCode
			NUM_HID_DESCRIPTORS,//bNumDescriptorcs
			USB_HID_DESCRIPTOR_REPORT,//bDescriptorType
			0x00,//wDescriptorLength LSB - total size of report descriptor
			0x00,//wDescriptorLength MSB - total size of report descriptor

			//Add here if you want to add physical descriptor
			//Don't forget to update the length of the HID descriptor as well as the length of the config descirptor!

			//Endpoint descriptor
			0x07,			// Endpoint Descriptor Length
			USB_ENDPOINT_DESCRIPTOR,		// Descriptor Type
			USB_ENDPOINT_DIRECTION_IN (1) | EP1,			// EP #	and direction
			EP_INT,		// Endpoint Type
			LOBYTE (EP1_MAX_PACKET_SIZE),			// Maximum packet size (LSB)
			HIBYTE (EP1_MAX_PACKET_SIZE),			// Maximum packet size (MSB)
			EP1_POLLING_INTERVAL,			// Polling Interval
	};


//
// The Configuration Descriptor
//
UCHAR gTheConfigDescriptorWithoutINEndpoint[] =
	{
	0x09,			// Configuration Descriptor Length
	CONFIG_DESCR,	// Descriptor Type
	0x09 + 0x09 + 0x09,	// Configuration + Iface + HID + EPs length (LSB)
	0x00,			// Configuration+EPs length (MSB)
	0x01,			// # of interfaces
	0x01,			// configurationValue
	0x00,			// Configuration String
	0x40,			// Attributes: Self power and RWU
	0x00,     // max Power

    // Interface descriptor
    0x09, // length
    USB_INTERFACE_DESCRIPTOR,	// descriptorType
    INTERFACE_FOR_HID,		        // interfaceNumber
    ALTSETTING_FOR_HID,		        // alternateSetting
    0,		        // numEndpoints
    USB_HID_CLASS,		        // interfaceClass
    USB_HID_SUBCLASS_NONE,		        // interfaceSubClass
    USB_HID_PROTOCOL_NONE,		        // interfaceProtocol
    0,		        // interface

			//HID Descriptor
			0x09,//bLength
			USB_HID_DESCRIPTOR_HID,//bDescriptorType
			0x10, 0x01,//bcdHID - class spec. release
			USB_HID_COUNTRY_NONE,//bCountryCode
			NUM_HID_DESCRIPTORS,//bNumDescriptorcs
			USB_HID_DESCRIPTOR_REPORT,//bDescriptorType
			0x00,//wDescriptorLength LSB - total size of report descriptor
			0x00,//wDescriptorLength MSB - total size of report descriptor

			//Add here if you want to add physical descriptor
			//Don't forget to update the length of the HID descriptor as well as the length of the config descirptor!

	};


/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/


/** private data **/
static PGETREPORTPROC gpOnGetReport;
static PSETREPORTPROC gpOnSetReport;
static PREPORTSENTPROC gpOnReportSent;
static PNAKSENTPROC gpOnNakSent;
static PSUSPENDPROC gpOnSuspend;
static PRESUMEPROC gpOnResume;
static PRESETPROC gpOnReset;

static bool gUseINEP;
static void *gpTheReportDescriptor;
static uint16 gTheReportDescriptorSize;

/** public functions **/

/** private functions **/

/** callback functions **/
static bool OnSetupRequest (UCHAR endpointNumber, PUSB_SETUP_DATA pSetupData, void *buffer, 
	PUINT pNumBytesToTransfer);
static bool OnGetDeviceDescriptor (void *buffer, PUINT pNumBytesToTransfer);
static bool OnGetConfigDescriptor (void *buffer, PUINT pNumBytesToTransfer);
static bool OnGetStringDescriptor (UCHAR stringIndex, USHORT langID, void *buffer, PUINT pNumBytesToTransfer);
static void OnTransferComplete (UCHAR endpointNumber);
static void OnNAKSent (UCHAR endpointNumber);
static bool OnConfigRequest (UCHAR newConfiguration, UCHAR newInterface, UCHAR newAltSetting);
static void OnSuspend (void);
static void OnResume (void);
static void OnReset (void);




/*------------------------------------------------------------
*	HIDCLASS_Init
*
*	Parameters:  
*
*	Globals Used:
*		gpOnGetReport, gpOnSetReport, gpOnReportSent, gpOnNakSent, gpOnSetReport
*		gpOnResume, gpOnReset, gUseINEP, gpTheReportDescriptor, gTheReportDescriptorSize
*		gTheDeviceDescriptor, gTheConfigDescriptorWithoutINEndpoint, gTheConfigDescriptorWithINEndpoint
*
*	Description:
*		This is the main initialization routine. The client provides the vendor id, device id,
*		revision, information about the Report descriptor, whether or not to use IN endpoint,
*		and a bunch of pointers to routines to be calleld under various conditions.
*
*	Returns:
*		true on success, false on failure.
*
*------------------------------------------------------------*/
bool HIDCLASS_Init (uint16 vid, uint16 pid, uint16 rel, bool useINEP, PUCHAR pReportDescriptor, uint16 reportDescriptorSize,
	PGETREPORTPROC pOnGetReportProc, PSETREPORTPROC pOnSetReportProc,
	PREPORTSENTPROC pOnReportSentProc, PNAKSENTPROC pOnNAKSentProc, PSUSPENDPROC pOnSuspendProc, 
	PRESUMEPROC pOnResumeProc, PRESETPROC pOnResetProc)
	{
		//Save the callback addresses
		gpOnGetReport = pOnGetReportProc;
		gpOnSetReport = pOnSetReportProc;
		gpOnReportSent = pOnReportSentProc;
		gpOnNakSent = pOnNAKSentProc;
		gpOnSuspend = pOnSuspendProc;
		gpOnResume = pOnResumeProc;
		gpOnReset = pOnResetProc;

		gUseINEP = useINEP;

		gpTheReportDescriptor = pReportDescriptor;
		gTheReportDescriptorSize = reportDescriptorSize;

		//Update the VID and PID
		memcpy (&gTheDeviceDescriptor[OFFSET_VENDOR_ID], &vid, sizeof (uint16));
		memcpy (&gTheDeviceDescriptor[OFFSET_PRODUCT_ID], &pid, sizeof (uint16));
		memcpy (&gTheDeviceDescriptor[OFFSET_RELEASE], &rel, sizeof (uint16));


		//Things to do before setting up endpoints
		if (!USBDriver_PreInit ())
			return false;

		//Are we using IN EP?
		if (useINEP)
			{
				memcpy (&gTheConfigDescriptorWithINEndpoint[OFFSET_REPORT_DESCRIPTOR_LENGTH],
					&reportDescriptorSize, sizeof (uint16));

				USBDriver_SetupEndpoint (EP1, ALTSETTING_FOR_HID, 
					INTERFACE_FOR_HID, EP_CONFIG1, EP_IN,
					EP_INT, EP1_MAX_PACKET_SIZE);
			}
		else
			memcpy (&gTheConfigDescriptorWithoutINEndpoint[OFFSET_REPORT_DESCRIPTOR_LENGTH],
				&reportDescriptorSize, sizeof (uint16));

		return USBDriver_CompleteInit (OnSetupRequest, OnGetDeviceDescriptor, 
			OnGetConfigDescriptor, OnGetStringDescriptor, 
			OnTransferComplete, OnNAKSent, 
			OnSuspend, OnResume, OnReset, OnConfigRequest, CONTROL_PIPE_MAX_TRANSFER_LENGTH);

	}


/*------------------------------------------------------------
*	HIDCLASS_PrepareReportEndpoint
*
*	Parameters:  
*		pReportData - points to a buffer containing the report data to provide
*			to the host, when the host requests it!
*		reportSize - the size of the report data in bytes.
*
*	Globals Used:
*		None
*
*	Description:
*		Prepares the Report Endpoint (IN) so that it can handle
*		and IN request from the host. It request usbdriver to
*		allocate a piece of DMA buffer - once it gets the address
*		to this buffer, it fills in the report data, and then
*		arms the endpoint.
*
*	Returns:
*		Nothing
*
*------------------------------------------------------------*/
void HIDCLASS_PrepareReportEndpoint (PUCHAR pReportData, uint16 reportSize)
	{
		char *pReportDataDMA = nil;

		ActivateEndpoint (EP1, EP_INT, EP1_MAX_PACKET_SIZE, reportSize);

		if (PrepareINEPForXFer (EP1, &pReportDataDMA))
			{
				memcpy (pReportDataDMA, pReportData, reportSize);
				//Arm the endpoint!
				ArmEndpoint (EP1, EP_IN);
			}

	}



/*------------------------------------------------------------
*	OnSetupRequest
*
*	Parameters:  
*		endpointNumber - the endpoint on which this request was received
*		pSetupData - points to a structure cantaining the SETUP DATA(0)
*		buffer - points to a buffer where the setup data is available, or should be placed.
*		pNumBytesToTransfer - points to a location where the number of bytes
*			to be sent back should be stored.
*	Globals Used:
*		None
*
*	Description:
*		This routine is called by the USB driver to handle a special setup request, 
*		that is not a GET DEVICE, GET CONFIG, or GET STRING descriptor
*		
*		Based on the request type, this routine reads/writes to the buffer
*		provided.
*
*	Returns:
*		true to succeed the request, false to stall the request
*
*------------------------------------------------------------*/
bool OnSetupRequest (UCHAR endpointNumber, PUSB_SETUP_DATA pSetupData, void *buffer, 
	PUINT pNumBytesToTransfer)
	{
		bool retVal = false; //stall


		if ((endpointNumber == EP0) && 
			(USB_CONTROL_REQUEST_GET_DIRECTION (pSetupData->bmRequestType) == USB_CONTROL_REQUEST_TYPE_IN) &&
			(USB_CONTROL_REQUEST_GET_TYPE (pSetupData->bmRequestType) == USB_CONTROL_REQUEST_TYPE_STANDARD) &&
			(USB_CONTROL_REQUEST_GET_RECIPIENT (pSetupData->bmRequestType) == USB_CONTROL_REQUEST_TYPE_INTERFACE) &&
			(pSetupData->wIndex == INTERFACE_FOR_HID) &&
			(pSetupData->bRequest == USB_GET_DESCRIPTOR)
			)
			{
				switch (HIBYTE(pSetupData->wValue))
					{
						case USB_HID_DESCRIPTOR_REPORT:
							if (LOBYTE(pSetupData->wValue) < NUM_HID_DESCRIPTORS)
								{
									Dprintf ("Get USB_HID_DESCRIPTOR_REPORT");
									//Here, we should return the appropriate report descriptor
									if (*pNumBytesToTransfer < gTheReportDescriptorSize)
										memcpy (buffer, gpTheReportDescriptor, *pNumBytesToTransfer);
									else
										{
											memcpy (buffer, gpTheReportDescriptor, gTheReportDescriptorSize);
											*pNumBytesToTransfer = gTheReportDescriptorSize;
										}
									retVal = true;
								}
							else
								Dprintf ("Unknown descriptor index requested!");
							break;

						//Other descriptor types can be handled here?
						default:
							Dprintf ("Unknown descriptor type requested!");
					}
			}
		else if ((endpointNumber == EP0) && 
			(USB_CONTROL_REQUEST_GET_DIRECTION (pSetupData->bmRequestType) == USB_CONTROL_REQUEST_TYPE_IN) &&
			(USB_CONTROL_REQUEST_GET_TYPE (pSetupData->bmRequestType) == USB_CONTROL_REQUEST_TYPE_CLASS) &&
			(USB_CONTROL_REQUEST_GET_RECIPIENT (pSetupData->bmRequestType) == USB_CONTROL_REQUEST_TYPE_INTERFACE) &&
			(pSetupData->bRequest == USB_HID_GET_REPORT_REQUEST)
			)
			{
				Dprintf ("GetReportRequest");
				if (gpOnGetReport)
					retVal = (gpOnGetReport) (pSetupData->wValue, buffer, pNumBytesToTransfer);
			}
		else if ((endpointNumber == EP0) && 
			(USB_CONTROL_REQUEST_GET_DIRECTION (pSetupData->bmRequestType) == USB_CONTROL_REQUEST_TYPE_OUT) &&
			(USB_CONTROL_REQUEST_GET_TYPE (pSetupData->bmRequestType) == USB_CONTROL_REQUEST_TYPE_CLASS) &&
			(USB_CONTROL_REQUEST_GET_RECIPIENT (pSetupData->bmRequestType) == USB_CONTROL_REQUEST_TYPE_INTERFACE) &&
			(pSetupData->bRequest == USB_HID_SET_REPORT_REQUEST)
			)
			{
				Dprintf ("SetReportRequest");
				if (gpOnSetReport)
					retVal = (gpOnSetReport) (pSetupData->wValue, buffer, pSetupData->wLength);
			}
		else
			{
				Dprintf1 ("Unknown setup request: %02x", pSetupData->bmRequestType);
				Dprintf3 ("Dir = %02x, Type = %02x, Rec = %02x", USB_CONTROL_REQUEST_GET_DIRECTION (pSetupData->bmRequestType),
					USB_CONTROL_REQUEST_GET_TYPE (pSetupData->bmRequestType),
					USB_CONTROL_REQUEST_GET_RECIPIENT (pSetupData->bmRequestType));
				Dprintf1 ("bRequest = %02x", pSetupData->bRequest);
			}
			

		return (retVal);
	}



/*------------------------------------------------------------
*	OnGetDeviceDescriptor
*
*	Parameters:  
*		buffer - location (DMA buffer) where the descriptor data is to
*			be put.
*		pNumBytesToTransfer - points to a location where the original wLength
*			is stored. This routine should update this location to indicate
*			the actual number of bytes transferred to the buffer.
*
*	Globals Used:
*		gTheDeviceDescriptor
*
*	Description:
*		This routine is in response to a GET DEVICE DESCRIPTOR
*		setup request. This routine fills in the required data
*
*	Returns:
*		true - always!
*
*------------------------------------------------------------*/
bool OnGetDeviceDescriptor (void *buffer, PUINT pNumBytesToTransfer)
	{
		if (*pNumBytesToTransfer < sizeof (gTheDeviceDescriptor))
			memcpy (buffer, &gTheDeviceDescriptor, *pNumBytesToTransfer);
		else
			{
				memcpy (buffer, &gTheDeviceDescriptor, sizeof (gTheDeviceDescriptor));
				*pNumBytesToTransfer = sizeof (gTheDeviceDescriptor);
			}

		return(true);
	}



/*------------------------------------------------------------
*	OnGetConfigDescriptor
*
*	Parameters:  
*		buffer - location (DMA buffer) where the descriptor data is to
*			be put.
*		pNumBytesToTransfer - points to a location where the original wLength
*			is stored. This routine should update this location to indicate
*			the actual number of bytes transferred to the buffer.
*
*	Globals Used:
*		gTheConfigDescriptor
*
*	Description:
*		This routine is in response to a GET CONFIG DESCRIPTOR
*		setup request. This routine fills in the required data
*
*	Returns:
*		true - always!
*
*------------------------------------------------------------*/
bool OnGetConfigDescriptor (void *buffer, PUINT pNumBytesToTransfer)
	{
		if (gUseINEP)
			{
				if (*pNumBytesToTransfer < sizeof(gTheConfigDescriptorWithINEndpoint))
					memcpy (buffer, &gTheConfigDescriptorWithINEndpoint, *pNumBytesToTransfer);
				else
					{
						memcpy (buffer, &gTheConfigDescriptorWithINEndpoint, sizeof (gTheConfigDescriptorWithINEndpoint));
						*pNumBytesToTransfer = sizeof (gTheConfigDescriptorWithINEndpoint);
					}
			}
		else
			{
				if (*pNumBytesToTransfer < sizeof (gTheConfigDescriptorWithoutINEndpoint))
					memcpy (buffer, &gTheConfigDescriptorWithoutINEndpoint, *pNumBytesToTransfer);
				else
					{
						memcpy (buffer, &gTheConfigDescriptorWithoutINEndpoint, 
							sizeof (gTheConfigDescriptorWithoutINEndpoint));
						*pNumBytesToTransfer = sizeof (gTheConfigDescriptorWithoutINEndpoint);
					}
			}
		return (true);
	}



/*------------------------------------------------------------
*	OnGetStringDescriptor
*
*	Parameters:  
*		wValue - the string to be retrieved
*		buffer - location (DMA buffer) where the descriptor data is to
*			be put.
*		pNumBytesToTransfer - points to a location where the original wLength
*			is stored. This routine should update this location to indicate
*			the actual number of bytes transferred to the buffer.
*
*	Globals Used:
*		Not Yet!
*
*	Description:
*		This routine is in response to a GET STRING DESCRIPTOR
*		setup request. This routine fills in the required data
*
*	Returns:
*		true - always!
*
*------------------------------------------------------------*/
bool OnGetStringDescriptor (UCHAR stringIndex, USHORT langID, void *buffer, PUINT pNumBytesToTransfer)
	{
		bool retVal = false;
		UINT neededSpace = 0;
		UINT maxBytesToTransfer = 0;
		UINT outIndex = 0, charIndex = 0;

		if (stringIndex == 0)
			{
				//Need to return a langid array here!
				if (*pNumBytesToTransfer < sizeof (gTheLangIDArray))
					memcpy (buffer, &gTheLangIDArray, *pNumBytesToTransfer);
				else
					{
						memcpy (buffer, &gTheLangIDArray, sizeof (gTheLangIDArray));
						*pNumBytesToTransfer = sizeof (gTheLangIDArray);
					}
				retVal = true;
			}
		else if ((stringIndex < TOTAL_STRING_DESCRIPTORS) && (stringIndex > 0))
			{
				//Convert this to a zero based index
				stringIndex--;
				neededSpace = (strlen (gTheStringDescriptors[stringIndex]) + 1) * 2;
				if (neededSpace > *pNumBytesToTransfer)
					maxBytesToTransfer = *pNumBytesToTransfer;
				else
					maxBytesToTransfer = *pNumBytesToTransfer = neededSpace;

				//Transfer as many as we can
				((char *)buffer)[outIndex++] = neededSpace;
				((char *)buffer)[outIndex++] = USB_STRING_DESCRIPTOR;
				while ((outIndex + 1) < maxBytesToTransfer)
					{
						((char *)buffer)[outIndex++] = gTheStringDescriptors[stringIndex][charIndex++];
						((char *)buffer)[outIndex++] = 0;
					}


				retVal = true;
			}

		//Don't have any strings yet!
		return(retVal);
	}




/*------------------------------------------------------------
*	OnConfigRequest
*
*	Parameters:  
*		newConfiguration - the new configuration after Config request.
*		newInterface - the new interface after Config(Set Interface) request.
*		newAltSetting - the new alt setting after Config(Set Interface) request
*
*	Globals Used:
*		gPrepareReport
*
*	Description:
*		This routien is called in response to a configuration change - i.e.
*		after receiving a Set Config or a Set Interface request.
*
*	Returns:
*		true - always
*
*------------------------------------------------------------*/
bool OnConfigRequest (UCHAR newConfiguration, UCHAR newInterface, UCHAR newAltSetting)
	{
		//Debug - we're currently removing the checking for alt setting!

		if ((newConfiguration == EP_CONFIG1))// && (newInterface == INTERFACE_FOR_HID) &&
			//(newAltSetting == ALTSETTING_FOR_HID))
			{
				//Looks like we're in, setup our IN endpoint if we want to!
				Dprintf ("HID Device Configured!");


				//Treat this as if a transfer had just completed!
				if (gpOnReportSent)
					(gpOnReportSent)();

			}
		else
			{
				//We should disarm the endpoint(s) at this point!
				Dprintf3 ("HID Dev Config. Inactive: %02x, %02x, %02x!", newConfiguration, newInterface, newAltSetting);
			}
			
		return true;
	}



/*------------------------------------------------------------
*	OnTransferComplete
*
*	Parameters:  
*		endpointNumber - the endpoint on which the transfer was completed.
*
*	Globals Used:
*		gPrepareReport
*
*	Description:
*		This routine is called when a transfer is completed on one of the
*		endpoints. In turn, this calls the hidclass client to let it know
*		about the transfer being complete.
*
*	Returns:
*		Nothing
*
*------------------------------------------------------------*/
void OnTransferComplete (UCHAR endpointNumber)
	{
		if (endpointNumber == EP1)
			{
				Dprintf ("OnINTransferComplete");
				if (gpOnReportSent)
					(gpOnReportSent)();
			}
	}




/*------------------------------------------------------------
*	OnNAKSent
*
*	Parameters:  
*		endpointNumber - endpoint where the NAK was sent.
*
*	Globals Used:
*		gPrepareReport
*
*	Description:
*		This routine is called when the device has sent a NAK in response
*		to an IN or OUT request while the DMA buffer for that endpoint
*		has been emptied. Please note that this will only be called in
*		response to an "expected" direction request - IN request for
*		endpoints setup as IN endpoints and OUT request for endpoints
*		setup as OUT endpoints. In turn, this calls the hidclass client
*		to let it know about the NAK.
*
*	Returns:
*		Nothing
*
*------------------------------------------------------------*/
void OnNAKSent (UCHAR endpointNumber)
	{
		if (endpointNumber == EP1)
			{
				if (gpOnNakSent)
					(gpOnNakSent)();
			}
	}




/*------------------------------------------------------------
*	OnSuspend
*
*	Parameters:  
*
*	Globals Used:
*		gpOnSuspend
*
*	Description:
*		This routine will be called when USBDriver detects a
*		suspend condition. Currently, it simply forwards
*		the call to the client!
*
*	Returns:
*		Nothing.
*
*------------------------------------------------------------*/
void OnSuspend (void)
	{
		if (gpOnSuspend)
			(gpOnSuspend)();
	}



/*------------------------------------------------------------
*	OnResume
*
*	Parameters:  
*
*	Globals Used:
*		gpOnResume
*
*	Description:
*		This routine will be called when the USBDriver detects that
*		the bus is out of suspend. Currently, it simply forwards
*		the call to the client!
*
*	Returns:
*		Nothing.
*
*------------------------------------------------------------*/
void OnResume (void)
	{
		if (gpOnResume)
			(gpOnResume) ();
	}


/*------------------------------------------------------------
*	OnReset
*
*	Parameters:  
*
*	Globals Used:
*		gpOnReset
*
*	Description:
*		This routine will be called when the USBDriver detects reset
*		signalling on the bus. Currently, it simply forwards
*		the call to the client!
*
*	Returns:
*		Nothing.
*
*------------------------------------------------------------*/
void OnReset (void)
	{
		if (gpOnReset)
			(gpOnReset)();
	}


/*----------------------------------------------------------------------------------
* $Log: HidClass.c,v $
* Revision 1.3  2002/10/30 02:36:40  Devendra
* - Added more abstraction to HIDCLASS. So that it's possible to build
*   different devices without making any changes to the HIDCLASS module.
* - Added proper support for handling USB SUSPEND, RESUME, and RESET events.
*
* Revision 1.2  2002/10/22 17:50:54  Devendra
* - Changed Product ID to 0x0005
*
* Revision 1.1  2002/10/22 17:23:38  Devendra
* Rearranged file locations.
*
* Revision 1.6  2002/10/14 05:16:48  Devendra
* Updated version number.
*
* Revision 1.5  2002/10/14 05:16:04  Devendra
* - Custom Device now using Feature reports for I/O
* - Added compiler define/project configuration to switch between mouse and custom device.
*
* Revision 1.4  2002/10/14 01:33:15  Devendra
* Got "custom" device functional with buttons and lights!
*
* Revision 1.3  2002/10/09 17:09:32  Devendra
* - Added support for handling IN endpoints.
* - Added HID descriptors (HID and HID Report)
* - Modified the Interface descriptors to comply to HID Class Spec.
* - Added functionality to use the push-buttons on the Eagle-35 as a mouse (for buttons as well as for movement).
* - The device is now a fully functioning USB Mouse!
*
* Revision 1.2  2002/10/08 19:57:02  Devendra
* - Added specific callbacks for getting Device, Config, and String descriptors.
* - BugFix: Max. Packet Size was not being set correctly in ArmEndpoint.
* - Added internal buffer based logging to allow real-time "printfs".
* - Added String descriptors.
*
* Revision 1.1  2002/09/20 06:51:30  Devendra
* First Rev.
*
*
*---------------------------------------------------------------------------------*/
